/*
 * Partitionability Checker
 *
 * by M.Hachimori, July 2006
 * revised: May 2023
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "s_complex.h"

void error();

int main(argc, argv)
  int argc;
  char *argv[];
{
  Simplices *faces, *tmp, *facets, *tmp_facets;
  int i, k, flag, dim, result, purity;
  int f_vec[MAXDIM+1], h_vec[MAXDIM+1];
  char filename[20];


  if (argc <= 1 || strcmp(argv[1], "-h") == 0
                || strcmp(argv[1], "--help") == 0
                || argc >= 4)
    error();


  if (argc == 3) 
    strcpy(filename, argv[2]);
  else 
    strcpy(filename, argv[1]);
    


  tmp = input_faces(filename);
  if (tmp == NULL) 
    error();
  sort_faces(tmp);
  faces = facets2faces(tmp);
  delete_faces(tmp);

  facets = faces2facets(faces);
  delete_faces(faces);

  /* check dimension and purity */
  purity = 1;
  dim = facets->vert[0];
  tmp_facets = facets;
  while(tmp_facets != NULL) {
    if (tmp_facets->vert[0] != dim) {
      if (tmp_facets->vert[0] > dim) dim = tmp_facets->vert[0];
      purity = 0;
      }
    tmp_facets = tmp_facets->next;
    }



  if (argc == 2) {  /* when no options are given */
    if ((dim == 2 || dim == 3) && purity == 1) {
      if (dim == 2) 
        result = is_partitionable2(facets);
      else 
        result = is_partitionable3(facets);
      }
    else if (dim <= 3) 
      result = is_partitionable_nonpure3(facets);
    else if (dim <= MAXDIM) 
      result = is_partitionable_nonpure_d(facets);
    else {
      printf("The dimension of your data is over MAXDIM.\n");
      printf("Please recompile the program with larger MAXDIM value.\n");
      }
    }
  else {  /* when option is given */
    if (dim <= MAXDIM) {
      if (strcmp(argv[1], "-2pure") == 0) 
        result = is_partitionable2(facets);
      else if (strcmp(argv[1], "-3pure") == 0)
        result = is_partitionable3(facets);
      else if (strcmp(argv[1], "-3") == 0)
        result = is_partitionable_nonpure3(facets);
      else if (strcmp(argv[1], "-dnonpure") == 0)
        result = is_partitionable_nonpure_d(facets);
      else 
        error();
      }
    else  {
      printf("The dimension of your data is over MAXDIM.\n");
      printf("Please recompile the program with larger MAXDIM value.\n");
      }
    }

  switch (result) {
    case 1:
      printf("partitionable\n");
      break;
    case 0:
      printf("not partitionable\n");
      break;
    case -1:
      printf("error\n");
      break;
    }


  delete_faces(facets);

  return(0);

}


void error() {

  printf("Usage:");
  printf("p_checker [option] <file> \n");
  printf("\n");
  printf("option:\n");
  printf("  -2pure : force using function for pure 2-dimensional cases \n");
  printf("  -3pure : force using function for pure 3-dimensional cases \n");
  printf("  -3 : force using function for pure and nonpure cases up to dimension 3 \n");
  printf("  -dnonpure : force using function for pure and nonpure cases for any dimension \n");
  exit(0);

}

